'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

/**
 * @author: Cuttle Cong、chuyik
 * @forker: shiftj18
 * @date: 2024/10/14
 * @description:
 */
var match = require('./utils/matchRule');

function isIgnore(opts, name) {
  return opts && opts.ignore && match(opts.ignore, name);
}

module.exports = function (babel) {
  var t = babel.types;
  return {
    visitor: {
      Program: {
        enter: function enter(path) {
          var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
              opts = _ref.opts;

          var bindings = path.scope.bindings;
          var UnRefBindings = new Map();

          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = Object.entries(bindings)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var _step$value = _slicedToArray(_step.value, 2),
                  name = _step$value[0],
                  binding = _step$value[1];

              if (!binding.path.parentPath || binding.kind !== 'module') continue;

              var source = binding.path.parentPath.get('source');
              var importName = source.node.value;
              if (!t.isStringLiteral(source) || isIgnore(opts, importName)) continue;

              var key = importName + '(' + (source.node.loc && source.node.loc.start.line) + ')';
              if (!UnRefBindings.has(key)) {
                UnRefBindings.set(key, binding);
              }

              if (binding.referenced) {
                UnRefBindings.set(key, null);
              } else {
                var nodeType = binding.path.node.type;
                if (nodeType === 'ImportSpecifier') {
                  binding.path.remove();
                } else if (nodeType === 'ImportDefaultSpecifier') {
                  binding.path.remove();
                } else if (nodeType === 'ImportNamespaceSpecifier') {
                  binding.path.remove();
                } else if (binding.path.parentPath) {
                  binding.path.parentPath.remove();
                }
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          UnRefBindings.forEach(function (binding, key) {
            if (binding && binding.path.parentPath) {
              binding.path.parentPath.remove();
            }
          });

          path.traverse({
            CallExpression: function CallExpression(callPath) {
              // 检查是否是 require()
              if (t.isIdentifier(callPath.node.callee, { name: 'require' })) {
                var source = callPath.node.arguments[0];
                if (t.isStringLiteral(source) && !isIgnore(opts, source.value)) {
                  if (t.isVariableDeclarator(callPath.parent)) {
                    if (t.isIdentifier(callPath.parent.id)) {
                      // 情况 const a = require('a');
                      var objName = callPath.parent.id && callPath.parent.id.name;
                      if (objName && !bindings[objName].referenced) {
                        callPath.parentPath.parentPath.remove();
                      }
                    } else if (t.isObjectPattern(callPath.parent.id)) {
                      // 情况 const { c1, c2 } = require('c');
                      // 情况 const { d1: rename, d2 } = require('c');
                      var keys = callPath.parent.id.properties.map(function (it) {
                        return it.value.name;
                      });
                      if (keys.every(function (key) {
                        return !bindings[key].referenced;
                      })) {
                        callPath.parentPath.parentPath.remove();
                      }
                    }
                  }
                }
              }
            }
          });
        }
      }
    }
  };
};